/****************************************************************************
**
** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
**
** This file is part of the painting module of the Qt Toolkit.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "qpictureformatplugin.h"
#ifndef QT_NO_PICTUREFORMATPLUGIN
#include "qpicture.h"

/*!
    \class QPictureFormatPlugin qpictureformatplugin.h
    \brief The QPictureFormatPlugin class provides an abstract base for custom picture format plugins.

    \ingroup plugins

    The picture format plugin is a simple plugin interface that makes
    it easy to create custom picture formats that can be used
    transparently by applications.

    Writing an picture format plugin is achieved by subclassing this
    base class, reimplementing the pure virtual functions keys() and
    installIOHandler(), and exporting the class with the
    Q_EXPORT_PLUGIN macro.  See the \link plugins-howto.html Plugins
    documentation\endlink for details.
*/

/*!
    \fn QStringList QPictureFormatPlugin::keys() const

    Returns the list of picture formats this plugin supports.

    \sa installIOHandler()
*/


/*!
    \fn  bool QPictureFormatPlugin::installIOHandler(const QString &format)

    Installs a QPictureIO picture I/O handler for the picture format \a
    format.

    \sa keys()
*/


/*!
    Constructs an picture format plugin with the given \a parent. This
    is invoked automatically by the Q_EXPORT_PLUGIN macro.
*/
QPictureFormatPlugin::QPictureFormatPlugin(QObject *parent)
    : QObject(parent)
{
}

/*!
    Destroys the picture format plugin.

    You never have to call this explicitly. Qt destroys a plugin
    automatically when it is no longer used.
*/
QPictureFormatPlugin::~QPictureFormatPlugin()
{
}


/*!\internal
 */
bool QPictureFormatPlugin::loadPicture(const QString &format, const QString &filename, QPicture *pic)
{
    Q_UNUSED(format)
    Q_UNUSED(filename)
    Q_UNUSED(pic)
    return false;
}

/*! \internal
 */
bool QPictureFormatPlugin::savePicture(const QString &format, const QString &filename, const QPicture &pic)
{
    Q_UNUSED(format)
    Q_UNUSED(filename)
    Q_UNUSED(pic)
    return false;
}

#endif // QT_NO_PICTUREFORMATPLUGIN
