/****************************************************************************
**
** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
**
** This file is part of the gui module of the Qt Toolkit.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef QCURSOR_P_H
#define QCURSOR_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of a number of Qt sources files.  This header file may change from
// version to version without notice, or even be removed.
//
// We mean it.
//

#include <qatomic.h>
#include <qglobal.h>
#include <qnamespace.h>

# if defined (Q_WS_MAC)
#  include <private/qt_mac_p.h>
#  if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3)
#    define QMAC_USE_BIG_CURSOR_API
#  endif
#  ifndef QMAC_NO_FAKECURSOR
     class QMacCursorWidget;
#  endif
   class QMacAnimateCursor;
# elif defined(Q_WS_X11)
#  include <private/qt_x11_p.h>
# elif defined(Q_WS_WIN)
#  include <qt_windows.h>
#endif

class QBitmap;
struct QCursorData {
    QCursorData(Qt::CursorShape s = Qt::ArrowCursor);
    ~QCursorData();

    static void initialize();
    static void cleanup();

    QAtomic ref;
    Qt::CursorShape cshape;
    QBitmap  *bm, *bmm;
    short     hx, hy;
#if defined (Q_WS_MAC) || defined(Q_WS_QWS)
    int id;
#endif
#if defined (Q_WS_WIN)
    HCURSOR hcurs;
#elif defined (Q_WS_X11)
    XColor fg, bg;
    Cursor hcurs;
    Pixmap pm, pmm;
#elif defined (Q_WS_MAC)
    enum { TYPE_None, TYPE_CursorImage, TYPE_CursPtr, TYPE_ThemeCursor, TYPE_FakeCursor, TYPE_BigCursor } type;
    union {
        struct {
            uint my_cursor:1;
            CursPtr   hcurs;
        } cp;
#ifndef QMAC_NO_FAKECURSOR
        struct {
            QMacCursorWidget *widget;
            CursPtr empty_curs;
        } fc;
#endif
#ifdef QMAC_USE_BIG_CURSOR_API
        char *big_cursor_name;
#endif
        CursorImageRec *ci;
        struct {
            QMacAnimateCursor *anim;
            ThemeCursor curs;
        } tc;
    } curs;
#endif
    static bool initialized;
};

#endif // QCURSOR_P_H
